@@ -13,7 +13,7 @@ class EnvVar(object):
 	IS_DEBUG = bool(os.environ.get('IS_DEBUG', default['IS_DEBUG']))
 	MODEL_DIR = os.environ.get("MODEL_DIR", default['MODEL_DIR'])
 	DEVICE = os.environ.get("DEVICE", default['DEVICE'])
-	BATCH_SIZE = int(os.environ.get("BATCH_SIZE", default['BATCH_SIZE']))
+	BATCH_SIZE = os.environ.get("BATCH_SIZE", default['BATCH_SIZE'])
 
 
 	@classmethod
